/*
 * dumpbb.c - Dump netbb format
 *
 * Written 2002 by Werner Almesberger, Caltech Netlab FAST project
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "dwarfx.h"


extern void open_elf(const char *name);
extern const void *mmap_file(const char *name,int *size);

int debug = 0;


/* ----- Argument processing ----------------------------------------------- */


static void usage(const char *name)
{
    fprintf(stderr,"usage: %s [-d] elf_file_name blackbox_file\n",name);
    fprintf(stderr,"%20s-d  enable debugging output\n","");
    exit(1);
}


int main(int argc,char **argv)
{
    const struct dwx_type *type;
    const void *data,*end;
    int c,size;

    while ((c = getopt(argc,argv,"d")) != EOF)
	switch (c) {
	    case 'd':
		debug = 1;
		break;
	    default:
		usage(*argv);
	}
    if (argc != optind+2) usage(*argv);
    open_elf(argv[optind]);
    while (1) {
	const void *p;

	p = dwarf_comp_unit();
	if (!p) break;
	dwx_read_types(p);
    }
    data = mmap_file(argv[optind+1],&size);
    end = data+size;
    type = dwx_lookup_type(dwx_struct,"sock",-1,0);
    if (!type) {
	fprintf(stderr,"struct sock not found\n");
	exit(1);
    }
    dwx_output_data(stdout,type,data);
    data += dwx_sizeof(type);
    type = dwx_lookup_type(dwx_struct,"sk_buff",-1,0);
    if (!type) {
	fprintf(stderr,"struct sk_buff not found\n");
	exit(1);
    }
    while (data != end) {
	unsigned long head,tail;

	if (data > end) {
	    fprintf(stderr,"dump file size error\n");
	    exit(1);
	}
	dwx_output_data(stdout,type,data);
	head = dwx_lookup_unsigned(type,data,"head");
	tail = dwx_lookup_unsigned(type,data,"tail");
	data += dwx_sizeof(type);
	dwx_output_hexdump(stdout,data,tail-head,0);
	data += tail-head;
    }
    return 0;
}
